@@ -15,9 +15,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #ifdef HAVE_CONFIG_H
@@ -61,12 +59,6 @@
 #  define UNIX_SOCKET "simple"
 #  define MODULE_ARGUMENTS "rate", "format", "channels", "sink", "source", "playback", "record",
 
-#  if defined(USE_TCP_SOCKETS)
-#    include "module-simple-protocol-tcp-symdef.h"
-#  else
-#    include "module-simple-protocol-unix-symdef.h"
-#  endif
-
   PA_MODULE_DESCRIPTION("Simple protocol "SOCKET_DESCRIPTION);
   PA_MODULE_USAGE("rate=<sample rate> "
                   "format=<sample format> "
@@ -83,12 +75,6 @@
 #  define UNIX_SOCKET "cli"
 #  define MODULE_ARGUMENTS
 
-#  ifdef USE_TCP_SOCKETS
-#    include "module-cli-protocol-tcp-symdef.h"
-#  else
-#   include "module-cli-protocol-unix-symdef.h"
-#  endif
-
   PA_MODULE_DESCRIPTION("Command line interface protocol "SOCKET_DESCRIPTION);
   PA_MODULE_USAGE(SOCKET_USAGE);
 #elif defined(USE_PROTOCOL_HTTP)
@@ -98,12 +84,6 @@
 #  define UNIX_SOCKET "http"
 #  define MODULE_ARGUMENTS
 
-#  ifdef USE_TCP_SOCKETS
-#    include "module-http-protocol-tcp-symdef.h"
-#  else
-#    include "module-http-protocol-unix-symdef.h"
-#  endif
-
   PA_MODULE_DESCRIPTION("HTTP "SOCKET_DESCRIPTION);
   PA_MODULE_USAGE(SOCKET_USAGE);
 #elif defined(USE_PROTOCOL_NATIVE)
@@ -113,21 +93,18 @@
 #  define UNIX_SOCKET PA_NATIVE_DEFAULT_UNIX_SOCKET
 #  define MODULE_ARGUMENTS_COMMON "cookie", "auth-cookie", "auth-cookie-enabled", "auth-anonymous",
 
-#  ifdef USE_TCP_SOCKETS
-#    include "module-native-protocol-tcp-symdef.h"
-#  else
-#    include "module-native-protocol-unix-symdef.h"
-#  endif
-
 #  if defined(HAVE_CREDS) && !defined(USE_TCP_SOCKETS)
-#    define MODULE_ARGUMENTS MODULE_ARGUMENTS_COMMON "auth-group", "auth-group-enable",
+#    define MODULE_ARGUMENTS MODULE_ARGUMENTS_COMMON "auth-group", "auth-group-enable", "srbchannel",
 #    define AUTH_USAGE "auth-group=<system group to allow access> auth-group-enable=<enable auth by UNIX group?> "
+#    define SRB_USAGE "srbchannel=<enable shared ringbuffer communication channel?> "
 #  elif defined(USE_TCP_SOCKETS)
 #    define MODULE_ARGUMENTS MODULE_ARGUMENTS_COMMON "auth-ip-acl",
 #    define AUTH_USAGE "auth-ip-acl=<IP address ACL to allow access> "
+#    define SRB_USAGE
 #  else
 #    define MODULE_ARGUMENTS MODULE_ARGUMENTS_COMMON
 #    define AUTH_USAGE
+#    define SRB_USAGE
 #    endif
 
   PA_MODULE_DESCRIPTION("Native protocol "SOCKET_DESCRIPTION);
@@ -135,6 +112,7 @@
                   "auth-cookie=<path to cookie file> "
                   "auth-cookie-enabled=<enable cookie authentication?> "
                   AUTH_USAGE
+                  SRB_USAGE
                   SOCKET_USAGE);
 #elif defined(USE_PROTOCOL_ESOUND)
 #  include <pulsecore/protocol-esound.h>
@@ -143,12 +121,6 @@
 #  define IPV4_PORT ESD_DEFAULT_PORT
 #  define MODULE_ARGUMENTS_COMMON "sink", "source", "auth-anonymous", "cookie", "auth-cookie", "auth-cookie-enabled",
 
-#  ifdef USE_TCP_SOCKETS
-#    include "module-esound-protocol-tcp-symdef.h"
-#  else
-#    include "module-esound-protocol-unix-symdef.h"
-#  endif
-
 #  if defined(USE_TCP_SOCKETS)
 #    define MODULE_ARGUMENTS MODULE_ARGUMENTS_COMMON "auth-ip-acl",
 #    define AUTH_USAGE "auth-ip-acl=<IP address ACL to allow access> "
