@@ -400,8 +400,9 @@ static void sink_set_mute_cb(pa_sink *s) {
 static void thread_func(void *userdata) {
     struct userdata *u = userdata;
     size_t offset = 0;
-    pa_usec_t last_timing;
-    uint32_t check_timing_count;
+    pa_usec_t last_timing = 0;
+    uint32_t check_timing_count = 1;
+    pa_usec_t intvl = 0;
 
     pa_assert(u);
 
@@ -413,7 +414,7 @@ static void thread_func(void *userdata) {
     for (;;) {
         struct pollfd *pollfd = NULL;
         unsigned int i, nbfds = 0;
-        pa_usec_t now, estimated, intvl;
+        pa_usec_t now, estimated;
         uint64_t position;
         size_t index;
         int ret;
@@ -634,6 +635,7 @@ static pa_device_port *raop_create_port(struct userdata *u, const char *server)
     pa_device_port_new_data_set_name(&data, "network-output");
     pa_device_port_new_data_set_description(&data, server);
     pa_device_port_new_data_set_direction(&data, PA_DIRECTION_OUTPUT);
+    pa_device_port_new_data_set_type(&data, PA_DEVICE_PORT_TYPE_NETWORK);
 
     port = pa_device_port_new(u->core, &data, 0);
 
