@@ -36,6 +36,19 @@
 
 #include "card.h"
 
+const char *pa_available_to_string(pa_available_t available) {
+    switch (available) {
+        case PA_AVAILABLE_UNKNOWN:
+            return "unknown";
+        case PA_AVAILABLE_NO:
+            return "no";
+        case PA_AVAILABLE_YES:
+            return "yes";
+        default:
+            pa_assert_not_reached();
+    }
+}
+
 pa_card_profile *pa_card_profile_new(const char *name, const char *description, size_t extra) {
     pa_card_profile *c;
 
@@ -70,13 +83,14 @@ void pa_card_profile_set_available(pa_card_profile *c, pa_available_t available)
 
     c->available = available;
     pa_log_debug("Setting card %s profile %s to availability status %s", c->card->name, c->name,
-                 available == PA_AVAILABLE_YES ? "yes" : available == PA_AVAILABLE_NO ? "no" : "unknown");
+                 pa_available_to_string(available));
 
     /* Post subscriptions to the card which owns us */
     pa_assert_se(core = c->card->core);
     pa_subscription_post(core, PA_SUBSCRIPTION_EVENT_CARD|PA_SUBSCRIPTION_EVENT_CHANGE, c->card->index);
 
-    pa_hook_fire(&core->hooks[PA_CORE_HOOK_CARD_PROFILE_AVAILABLE_CHANGED], c);
+    if (c->card->linked)
+        pa_hook_fire(&core->hooks[PA_CORE_HOOK_CARD_PROFILE_AVAILABLE_CHANGED], c);
 }
 
 pa_card_new_data* pa_card_new_data_init(pa_card_new_data *data) {
@@ -186,7 +200,9 @@ void pa_card_choose_initial_profile(pa_card *card) {
      * or if all profiles are unavailable, pick the profile with the highest
      * priority regardless of its availability. */
 
+    pa_log_debug("Looking for initial profile for card %s", card->name);
     PA_HASHMAP_FOREACH(profile, card->profiles, state) {
+        pa_log_debug("%s availability %s", profile->name, pa_available_to_string(profile->available));
         if (profile->available == PA_AVAILABLE_NO)
             continue;
 
@@ -204,6 +220,7 @@ void pa_card_choose_initial_profile(pa_card *card) {
 
     card->active_profile = best;
     card->save_profile = false;
+    pa_log_info("%s: active_profile: %s", card->name, card->active_profile->name);
 
     /* Let policy modules override the default. */
     pa_hook_fire(&card->core->hooks[PA_CORE_HOOK_CARD_CHOOSE_INITIAL_PROFILE], card);
@@ -318,6 +335,7 @@ int pa_card_set_profile(pa_card *c, pa_card_profile *profile, bool save) {
     if (c->linked && (r = c->set_profile(c, profile)) < 0)
         return r;
 
+    pa_log_debug("%s: active_profile: %s -> %s", c->name, c->active_profile->name, profile->name);
     c->active_profile = profile;
     c->save_profile = save;
 
@@ -325,7 +343,6 @@ int pa_card_set_profile(pa_card *c, pa_card_profile *profile, bool save) {
         update_port_preferred_profile(c);
 
     if (c->linked) {
-        pa_log_info("Changed profile of card %u \"%s\" to %s", c->index, c->name, profile->name);
         pa_hook_fire(&c->core->hooks[PA_CORE_HOOK_CARD_PROFILE_CHANGED], c);
         pa_subscription_post(c->core, PA_SUBSCRIPTION_EVENT_CARD|PA_SUBSCRIPTION_EVENT_CHANGE, c->index);
     }
