@@ -100,11 +100,7 @@ static void handle_io_event(pa_mainloop_api *ea, pa_io_event *e, int fd, pa_io_e
     unsigned int flags = 0;
     DBusWatch *watch = userdata;
 
-#ifdef HAVE_DBUS_WATCH_GET_UNIX_FD
     pa_assert(fd == dbus_watch_get_unix_fd(watch));
-#else
-    pa_assert(fd == dbus_watch_get_fd(watch));
-#endif
 
     if (!dbus_watch_get_enabled(watch)) {
         pa_log_warn("Asked to handle disabled watch: %p %i", (void*) watch, fd);
@@ -153,11 +149,7 @@ static dbus_bool_t add_watch(DBusWatch *watch, void *data) {
 
     ev = c->mainloop->io_new(
             c->mainloop,
-#ifdef HAVE_DBUS_WATCH_GET_UNIX_FD
             dbus_watch_get_unix_fd(watch),
-#else
-            dbus_watch_get_fd(watch),
-#endif
             get_watch_flags(watch), handle_io_event, watch);
 
     dbus_watch_set_data(watch, ev, NULL);
