@@ -63,6 +63,10 @@ void *pa_dynarray_steal_last(pa_dynarray *array);
 
 unsigned pa_dynarray_size(pa_dynarray *array);
 
+/* Returns -PA_ERR_NOENTITY if i is out of bounds, and zero otherwise.
+ * Here i is the location index in the array like 0, ..., array->entries */
+int pa_dynarray_insert_by_index(pa_dynarray *array, void *p, unsigned i);
+
 #define PA_DYNARRAY_FOREACH(elem, array, idx) \
     for ((idx) = 0; ((elem) = pa_dynarray_get(array, idx)); (idx)++)
 
