@@ -15,9 +15,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #ifdef HAVE_CONFIG_H
@@ -56,6 +54,10 @@ int deny_severity = LOG_WARNING;
 
 #endif /* HAVE_LIBWRAP */
 
+#ifdef HAVE_SYSTEMD_DAEMON
+#include <systemd/sd-daemon.h>
+#endif
+
 #include <pulse/xmalloc.h>
 #include <pulse/util.h>
 
@@ -74,6 +76,7 @@ struct pa_socket_server {
     PA_REFCNT_DECLARE;
     int fd;
     char *filename;
+    bool activated;
     char *tcpwrap_service;
 
     pa_socket_server_on_connection_cb_t on_connection;
@@ -82,7 +85,6 @@ struct pa_socket_server {
     pa_io_event *io_event;
     pa_mainloop_api *mainloop;
     enum {
-        SOCKET_SERVER_GENERIC,
         SOCKET_SERVER_IPV4,
         SOCKET_SERVER_UNIX,
         SOCKET_SERVER_IPV6
@@ -144,7 +146,7 @@ finish:
     pa_socket_server_unref(s);
 }
 
-pa_socket_server* pa_socket_server_new(pa_mainloop_api *m, int fd) {
+static pa_socket_server* socket_server_new(pa_mainloop_api *m, int fd) {
     pa_socket_server *s;
 
     pa_assert(m);
@@ -157,8 +159,6 @@ pa_socket_server* pa_socket_server_new(pa_mainloop_api *m, int fd) {
 
     pa_assert_se(s->io_event = m->io_new(m, fd, PA_IO_EVENT_INPUT, callback, s));
 
-    s->type = SOCKET_SERVER_GENERIC;
-
     return s;
 }
 
@@ -174,43 +174,63 @@ pa_socket_server* pa_socket_server_ref(pa_socket_server *s) {
 
 pa_socket_server* pa_socket_server_new_unix(pa_mainloop_api *m, const char *filename) {
     int fd = -1;
+    bool activated = false;
     struct sockaddr_un sa;
     pa_socket_server *s;
 
     pa_assert(m);
     pa_assert(filename);
 
-    if ((fd = pa_socket_cloexec(PF_UNIX, SOCK_STREAM, 0)) < 0) {
-        pa_log("socket(): %s", pa_cstrerror(errno));
-        goto fail;
+#ifdef HAVE_SYSTEMD_DAEMON
+    {
+        int n = sd_listen_fds(0);
+        if (n > 0) {
+            for (int i = 0; i < n; ++i) {
+                if (sd_is_socket_unix(SD_LISTEN_FDS_START + i, SOCK_STREAM, 1, filename, 0) > 0) {
+                    fd = SD_LISTEN_FDS_START + i;
+                    activated = true;
+                    pa_log_info("Found socket activation socket for '%s' \\o/", filename);
+                    break;
+                }
+            }
+        }
+    }
+#endif
+
+    if (fd < 0) {
+        if ((fd = pa_socket_cloexec(PF_UNIX, SOCK_STREAM, 0)) < 0) {
+            pa_log("socket(PF_UNIX): %s", pa_cstrerror(errno));
+            goto fail;
+        }
+
+        memset(&sa, 0, sizeof(sa));
+        sa.sun_family = AF_UNIX;
+        pa_strlcpy(sa.sun_path, filename, sizeof(sa.sun_path));
+
+        pa_make_socket_low_delay(fd);
+
+        if (bind(fd, (struct sockaddr*) &sa, (socklen_t) SUN_LEN(&sa)) < 0) {
+            pa_log("bind(): %s", pa_cstrerror(errno));
+            goto fail;
+        }
+
+        /* Allow access from all clients. Sockets like this one should
+        * always be put inside a directory with proper access rights,
+        * because not all OS check the access rights on the socket
+        * inodes. */
+        chmod(filename, 0777);
+
+        if (listen(fd, 5) < 0) {
+            pa_log("listen(): %s", pa_cstrerror(errno));
+            goto fail;
+        }
     }
 
-    memset(&sa, 0, sizeof(sa));
-    sa.sun_family = AF_UNIX;
-    pa_strlcpy(sa.sun_path, filename, sizeof(sa.sun_path));
-
-    pa_make_socket_low_delay(fd);
-
-    if (bind(fd, (struct sockaddr*) &sa, (socklen_t) SUN_LEN(&sa)) < 0) {
-        pa_log("bind(): %s", pa_cstrerror(errno));
-        goto fail;
-    }
-
-    /* Allow access from all clients. Sockets like this one should
-     * always be put inside a directory with proper access rights,
-     * because not all OS check the access rights on the socket
-     * inodes. */
-    chmod(filename, 0777);
-
-    if (listen(fd, 5) < 0) {
-        pa_log("listen(): %s", pa_cstrerror(errno));
-        goto fail;
-    }
-
-    pa_assert_se(s = pa_socket_server_new(m, fd));
+    pa_assert_se(s = socket_server_new(m, fd));
 
     s->filename = pa_xstrdup(filename);
     s->type = SOCKET_SERVER_UNIX;
+    s->activated = activated;
 
     return s;
 
@@ -229,57 +249,76 @@ pa_socket_server* pa_socket_server_new_unix(pa_mainloop_api *m, const char *file
 
 #endif /* HAVE_SYS_UN_H */
 
-pa_socket_server* pa_socket_server_new_ipv4(pa_mainloop_api *m, uint32_t address, uint16_t port, pa_bool_t fallback, const char *tcpwrap_service) {
+pa_socket_server* pa_socket_server_new_ipv4(pa_mainloop_api *m, uint32_t address, uint16_t port, bool fallback, const char *tcpwrap_service) {
     pa_socket_server *ss;
     int fd = -1;
+    bool activated = false;
     struct sockaddr_in sa;
     int on = 1;
 
     pa_assert(m);
     pa_assert(port);
 
-    if ((fd = pa_socket_cloexec(PF_INET, SOCK_STREAM, 0)) < 0) {
-        pa_log("socket(PF_INET): %s", pa_cstrerror(errno));
-        goto fail;
+#ifdef HAVE_SYSTEMD_DAEMON
+    {
+        int n = sd_listen_fds(0);
+        if (n > 0) {
+            for (int i = 0; i < n; ++i) {
+                if (sd_is_socket_inet(SD_LISTEN_FDS_START + i, AF_INET, SOCK_STREAM, 1, port) > 0) {
+                    fd = SD_LISTEN_FDS_START + i;
+                    activated = true;
+                    pa_log_info("Found socket activation socket for ipv4 in port '%d' \\o/", port);
+                    break;
+                }
+            }
+        }
     }
-
-#ifdef SO_REUSEADDR
-    if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (const void *) &on, sizeof(on)) < 0)
-        pa_log("setsockopt(): %s", pa_cstrerror(errno));
 #endif
 
-    pa_make_tcp_socket_low_delay(fd);
-
-    memset(&sa, 0, sizeof(sa));
-    sa.sin_family = AF_INET;
-    sa.sin_port = htons(port);
-    sa.sin_addr.s_addr = htonl(address);
-
-    if (bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
-
-        if (errno == EADDRINUSE && fallback) {
-            sa.sin_port = 0;
-
-            if (bind(fd, (struct sockaddr *) &sa, sizeof(sa)) >= 0)
-                goto good;
+    if (fd < 0) {
+        if ((fd = pa_socket_cloexec(PF_INET, SOCK_STREAM, 0)) < 0) {
+            pa_log("socket(PF_INET): %s", pa_cstrerror(errno));
+            goto fail;
         }
 
-        pa_log("bind(): %s", pa_cstrerror(errno));
-        goto fail;
+#ifdef SO_REUSEADDR
+        if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (const void *) &on, sizeof(on)) < 0)
+            pa_log("setsockopt(): %s", pa_cstrerror(errno));
+#endif
 
-    good:
-        ;
+        pa_make_tcp_socket_low_delay(fd);
+
+        memset(&sa, 0, sizeof(sa));
+        sa.sin_family = AF_INET;
+        sa.sin_port = htons(port);
+        sa.sin_addr.s_addr = htonl(address);
+
+        if (bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
+
+            if (errno == EADDRINUSE && fallback) {
+                sa.sin_port = 0;
+
+                if (bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
+                    pa_log("bind(): %s", pa_cstrerror(errno));
+                    goto fail;
+                }
+            } else {
+                pa_log("bind(): %s", pa_cstrerror(errno));
+                goto fail;
+            }
+        }
+
+        if (listen(fd, 5) < 0) {
+            pa_log("listen(): %s", pa_cstrerror(errno));
+            goto fail;
+        }
     }
 
-    if (listen(fd, 5) < 0) {
-        pa_log("listen(): %s", pa_cstrerror(errno));
-        goto fail;
-    }
+    pa_assert_se(ss = socket_server_new(m, fd));
 
-    if ((ss = pa_socket_server_new(m, fd))) {
-        ss->type = SOCKET_SERVER_IPV4;
-        ss->tcpwrap_service = pa_xstrdup(tcpwrap_service);
-    }
+    ss->type = SOCKET_SERVER_IPV4;
+    ss->tcpwrap_service = pa_xstrdup(tcpwrap_service);
+    ss->activated = activated;
 
     return ss;
 
@@ -291,64 +330,83 @@ fail:
 }
 
 #ifdef HAVE_IPV6
-pa_socket_server* pa_socket_server_new_ipv6(pa_mainloop_api *m, const uint8_t address[16], uint16_t port, pa_bool_t fallback, const char *tcpwrap_service) {
+pa_socket_server* pa_socket_server_new_ipv6(pa_mainloop_api *m, const uint8_t address[16], uint16_t port, bool fallback, const char *tcpwrap_service) {
     pa_socket_server *ss;
     int fd = -1;
+    bool activated = false;
     struct sockaddr_in6 sa;
     int on;
 
     pa_assert(m);
     pa_assert(port > 0);
 
-    if ((fd = pa_socket_cloexec(PF_INET6, SOCK_STREAM, 0)) < 0) {
-        pa_log("socket(PF_INET6): %s", pa_cstrerror(errno));
-        goto fail;
+#ifdef HAVE_SYSTEMD_DAEMON
+    {
+        int n = sd_listen_fds(0);
+        if (n > 0) {
+            for (int i = 0; i < n; ++i) {
+                if (sd_is_socket_inet(SD_LISTEN_FDS_START + i, AF_INET6, SOCK_STREAM, 1, port) > 0) {
+                    fd = SD_LISTEN_FDS_START + i;
+                    activated = true;
+                    pa_log_info("Found socket activation socket for ipv6 in port '%d' \\o/", port);
+                    break;
+                }
+            }
+        }
     }
+#endif
+
+    if (fd < 0) {
+        if ((fd = pa_socket_cloexec(PF_INET6, SOCK_STREAM, 0)) < 0) {
+            pa_log("socket(PF_INET6): %s", pa_cstrerror(errno));
+            goto fail;
+        }
 
 #ifdef IPV6_V6ONLY
-    on = 1;
-    if (setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, (const void *) &on, sizeof(on)) < 0)
-        pa_log("setsockopt(IPPROTO_IPV6, IPV6_V6ONLY): %s", pa_cstrerror(errno));
+        on = 1;
+        if (setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, (const void *) &on, sizeof(on)) < 0)
+            pa_log("setsockopt(IPPROTO_IPV6, IPV6_V6ONLY): %s", pa_cstrerror(errno));
 #endif
 
 #ifdef SO_REUSEADDR
-    on = 1;
-    if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (const void *) &on, sizeof(on)) < 0)
-        pa_log("setsockopt(SOL_SOCKET, SO_REUSEADDR, 1): %s", pa_cstrerror(errno));
+        on = 1;
+        if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (const void *) &on, sizeof(on)) < 0)
+            pa_log("setsockopt(SOL_SOCKET, SO_REUSEADDR, 1): %s", pa_cstrerror(errno));
 #endif
 
-    pa_make_tcp_socket_low_delay(fd);
+        pa_make_tcp_socket_low_delay(fd);
 
-    memset(&sa, 0, sizeof(sa));
-    sa.sin6_family = AF_INET6;
-    sa.sin6_port = htons(port);
-    memcpy(sa.sin6_addr.s6_addr, address, 16);
+        memset(&sa, 0, sizeof(sa));
+        sa.sin6_family = AF_INET6;
+        sa.sin6_port = htons(port);
+        memcpy(sa.sin6_addr.s6_addr, address, 16);
 
-    if (bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
+        if (bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
 
-        if (errno == EADDRINUSE && fallback) {
-            sa.sin6_port = 0;
+            if (errno == EADDRINUSE && fallback) {
+                sa.sin6_port = 0;
 
-            if (bind(fd, (struct sockaddr *) &sa, sizeof(sa)) >= 0)
-                goto good;
+                if (bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
+                    pa_log("bind(): %s", pa_cstrerror(errno));
+                    goto fail;
+                }
+            } else {
+                pa_log("bind(): %s", pa_cstrerror(errno));
+                goto fail;
+            }
         }
 
-        pa_log("bind(): %s", pa_cstrerror(errno));
-        goto fail;
-
-    good:
-        ;
+        if (listen(fd, 5) < 0) {
+            pa_log("listen(): %s", pa_cstrerror(errno));
+            goto fail;
+        }
     }
 
-    if (listen(fd, 5) < 0) {
-        pa_log("listen(): %s", pa_cstrerror(errno));
-        goto fail;
-    }
+    pa_assert_se(ss = socket_server_new(m, fd));
 
-    if ((ss = pa_socket_server_new(m, fd))) {
-        ss->type = SOCKET_SERVER_IPV6;
-        ss->tcpwrap_service = pa_xstrdup(tcpwrap_service);
-    }
+    ss->type = SOCKET_SERVER_IPV6;
+    ss->tcpwrap_service = pa_xstrdup(tcpwrap_service);
+    ss->activated = activated;
 
     return ss;
 
@@ -360,7 +418,7 @@ fail:
 }
 #endif
 
-pa_socket_server* pa_socket_server_new_ipv4_loopback(pa_mainloop_api *m, uint16_t port, pa_bool_t fallback, const char *tcpwrap_service) {
+pa_socket_server* pa_socket_server_new_ipv4_loopback(pa_mainloop_api *m, uint16_t port, bool fallback, const char *tcpwrap_service) {
     pa_assert(m);
     pa_assert(port > 0);
 
@@ -368,7 +426,7 @@ pa_socket_server* pa_socket_server_new_ipv4_loopback(pa_mainloop_api *m, uint16_
 }
 
 #ifdef HAVE_IPV6
-pa_socket_server* pa_socket_server_new_ipv6_loopback(pa_mainloop_api *m, uint16_t port, pa_bool_t fallback, const char *tcpwrap_service) {
+pa_socket_server* pa_socket_server_new_ipv6_loopback(pa_mainloop_api *m, uint16_t port, bool fallback, const char *tcpwrap_service) {
     pa_assert(m);
     pa_assert(port > 0);
 
@@ -376,7 +434,7 @@ pa_socket_server* pa_socket_server_new_ipv6_loopback(pa_mainloop_api *m, uint16_
 }
 #endif
 
-pa_socket_server* pa_socket_server_new_ipv4_any(pa_mainloop_api *m, uint16_t port, pa_bool_t fallback, const char *tcpwrap_service) {
+pa_socket_server* pa_socket_server_new_ipv4_any(pa_mainloop_api *m, uint16_t port, bool fallback, const char *tcpwrap_service) {
     pa_assert(m);
     pa_assert(port > 0);
 
@@ -384,7 +442,7 @@ pa_socket_server* pa_socket_server_new_ipv4_any(pa_mainloop_api *m, uint16_t por
 }
 
 #ifdef HAVE_IPV6
-pa_socket_server* pa_socket_server_new_ipv6_any(pa_mainloop_api *m, uint16_t port, pa_bool_t fallback, const char *tcpwrap_service) {
+pa_socket_server* pa_socket_server_new_ipv6_any(pa_mainloop_api *m, uint16_t port, bool fallback, const char *tcpwrap_service) {
     pa_assert(m);
     pa_assert(port > 0);
 
@@ -392,7 +450,7 @@ pa_socket_server* pa_socket_server_new_ipv6_any(pa_mainloop_api *m, uint16_t por
 }
 #endif
 
-pa_socket_server* pa_socket_server_new_ipv4_string(pa_mainloop_api *m, const char *name, uint16_t port, pa_bool_t fallback, const char *tcpwrap_service) {
+pa_socket_server* pa_socket_server_new_ipv4_string(pa_mainloop_api *m, const char *name, uint16_t port, bool fallback, const char *tcpwrap_service) {
     struct in_addr ipv4;
 
     pa_assert(m);
@@ -406,7 +464,7 @@ pa_socket_server* pa_socket_server_new_ipv4_string(pa_mainloop_api *m, const cha
 }
 
 #ifdef HAVE_IPV6
-pa_socket_server* pa_socket_server_new_ipv6_string(pa_mainloop_api *m, const char *name, uint16_t port, pa_bool_t fallback, const char *tcpwrap_service) {
+pa_socket_server* pa_socket_server_new_ipv6_string(pa_mainloop_api *m, const char *name, uint16_t port, bool fallback, const char *tcpwrap_service) {
     struct in6_addr ipv6;
 
     pa_assert(m);
@@ -423,10 +481,9 @@ pa_socket_server* pa_socket_server_new_ipv6_string(pa_mainloop_api *m, const cha
 static void socket_server_free(pa_socket_server*s) {
     pa_assert(s);
 
-    if (s->filename) {
+    if (!s->activated && s->filename)
         unlink(s->filename);
-        pa_xfree(s->filename);
-    }
+    pa_xfree(s->filename);
 
     pa_close(s->fd);
 
